import * as axios from "axios";
import { API } from "./API";
import { Options } from "./Options";
import { DataParser } from "./DataParser";
import { RendersContext, ServerContext, SkinContext, CapeContext, PlayerContext, SearchContext } from "./contexts";
import { IOptions, ITransformSkinOptions, ICheckServerLikeOptions, IFriend, IGetSkinsOptions, IGetSkinHistoryOptions, IRendersContextOptions, IContextOptions, Username, CapeHash, CapeName, CapeType, Model, Transformation, Sort, NameStatus } from "./interfaces";
import AxiosInstance = axios.AxiosInstance;
export declare class NameMC extends DataParser {
    /**
     * @hidden
     */
    readonly client: AxiosInstance;
    /**
     * Parameters passed to the constructor during class initialization
     */
    readonly options: Options;
    /**
     * Class for API Requests
     */
    readonly api: API;
    constructor(options?: IOptions);
    /**
     * Get skin history by nickname
     */
    skinHistory({ username, page }: IGetSkinHistoryOptions): Promise<SkinContext[]>;
    /**
     * Get player info by nickname
     */
    getPlayer(username: Username): Promise<PlayerContext>;
    /**
     * Get skin renders
     */
    getRenders(options: Omit<IRendersContextOptions, keyof IContextOptions>): RendersContext;
    /**
     * Transform skin method
     */
    transformSkin({ skin, transformation, model }: ITransformSkinOptions): Promise<SkinContext>;
    /**
     * Get cape info by cape hash
     */
    getCapeInfo(hash: CapeHash): CapeContext;
    /**
     * Get player friends by nickname
     */
    getFriends(username: Username): Promise<IFriend[]>;
    /**
     * Get new skins
     *
     * @see {@link https://namemc.com/minecraft-skins/new | NameMC New Skins}
     */
    getSkins(options: IGetSkinsOptions<"new", undefined, number>): Promise<SkinContext[]>;
    /**
     * Get skins tags
     *
     * @see {@link https://namemc.com/minecraft-skins/tag | NameMC Tagged Skins}
     */
    getSkins(options: IGetSkinsOptions<"tag", undefined, number>): Promise<SkinContext[]>;
    /**
     * Get skins by tag
     *
     * @see {@link https://namemc.com/minecraft-skins/tag/girl | NameMC Girl Tag Skins}
     */
    getSkins(options: IGetSkinsOptions<"tag", string, number>): Promise<SkinContext[]>;
    /**
     * Get daily trending skins
     *
     * @see {@link https://namemc.com/minecraft-skins/trending/daily | NameMC Daily Trending Skins}
     */
    getSkins(options: IGetSkinsOptions<"trending", "daily", number>): Promise<SkinContext[]>;
    /**
     * Get weekly trending skins
     *
     * @see {@link https://namemc.com/minecraft-skins/trending/weekly | NameMC Weekly Trending Skins}
     */
    getSkins(options: IGetSkinsOptions<"trending", "weekly", number>): Promise<SkinContext[]>;
    /**
     * Get monthly trending skins
     *
     * @see {@link https://namemc.com/minecraft-skins/trending/monthly | NameMC Monthly Trending Skins}
     */
    getSkins(options: IGetSkinsOptions<"trending", "monthly", number>): Promise<SkinContext[]>;
    /**
     * Get top trending skins
     *
     * @see {@link https://namemc.com/minecraft-skins/trending/top | NameMC Top Trending Skins}
     */
    getSkins(options: IGetSkinsOptions<"trending", "top", number>): Promise<SkinContext[]>;
    /**
     * Get random skins
     *
     * @see {@link https://namemc.com/minecraft-skins/random | NameMC Random Skins}
     */
    getSkins(options: IGetSkinsOptions<"random">): Promise<SkinContext[]>;
    /**
     * Get information about skin
     */
    getSkin(hash: SkinContext["hash"]): Promise<SkinContext>;
    /**
     * Get servers list
     */
    getServers(page?: number): Promise<ServerContext[]>;
    /**
     * Get server by ip
     */
    getServer(ip: string): Promise<ServerContext>;
    /**
     * Get server likes by ip
     */
    getServerLikes(ip: string): Promise<string[]>;
    /**
     * Check server like value by username
     */
    checkServerLike({ ip, username }: ICheckServerLikeOptions): Promise<boolean>;
    /**
     * Search
     */
    search(query: string): Promise<SearchContext | PlayerContext | ServerContext>;
    /**
     * @hidden
     */
    private addInterceptors;
}
export * from "./interfaces";
export { API, RendersContext, ServerContext, SkinContext, CapeContext, PlayerContext, SearchContext, Transformation, NameStatus, CapeHash, CapeName, CapeType, Model, Sort };
