"use strict";

var _exportNames = {
  NameMC: true,
  API: true,
  RendersContext: true,
  ServerContext: true,
  SkinContext: true,
  CapeContext: true,
  PlayerContext: true,
  SearchContext: true,
  CapeHash: true,
  CapeName: true,
  CapeType: true,
  Model: true,
  Transformation: true,
  Sort: true,
  NameStatus: true
};
exports.NameMC = void 0;

var axios = require("axios");

var _API = require("./API");

exports.API = _API.API;

var _Options = require("./Options");

var _DataParser = require("./DataParser");

var _WrapperError = require("./WrapperError");

var _contexts = require("./contexts");

exports.RendersContext = _contexts.RendersContext;
exports.ServerContext = _contexts.ServerContext;
exports.SkinContext = _contexts.SkinContext;
exports.CapeContext = _contexts.CapeContext;
exports.PlayerContext = _contexts.PlayerContext;
exports.SearchContext = _contexts.SearchContext;

var _utils = require("./utils");

var _interfaces = require("./interfaces");

exports.CapeHash = _interfaces.CapeHash;
exports.CapeName = _interfaces.CapeName;
exports.CapeType = _interfaces.CapeType;
exports.Model = _interfaces.Model;
exports.Transformation = _interfaces.Transformation;
exports.Sort = _interfaces.Sort;
exports.NameStatus = _interfaces.NameStatus;
Object.keys(_interfaces).forEach(function (key) {
  if (key === "default" || key === "__esModule") return;
  if (Object.prototype.hasOwnProperty.call(_exportNames, key)) return;
  if (key in exports && exports[key] === _interfaces[key]) return;
  exports[key] = _interfaces[key];
});
const DESKTOP_USER_AGENT = "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36";

class NameMC extends _DataParser.DataParser {
  constructor(options = {}) {
    super();
    this.options = new _Options.Options(options);
    this.api = new _API.API(); // @ts-ignore

    this.client = axios.create({
      baseURL: this.options.getEndpoint(),
      headers: typeof window === "undefined" ? {
        "User-Agent": DESKTOP_USER_AGENT
      } : {}
    });
    this.addInterceptors();
  }
  /**
   * Get skin history by nickname
   */


  async skinHistory({
    username,
    page = 1
  }) {
    const {
      uuid
    } = await this.getPlayer(username);
    return this.client.get(`/minecraft-skins/profile/${uuid}`, {
      params: {
        page
      }
    }).then(({
      data
    }) => {
      var _skins$;

      const skins = this.parseSkins(data);

      if ((_skins$ = skins[0]) !== null && _skins$ !== void 0 && _skins$.hash) {
        return skins;
      }

      throw new _WrapperError.WrapperError("NO_USEFUL");
    });
  }
  /**
   * Get player info by nickname
   */


  async getPlayer(username) {
    const context = new _contexts.PlayerContext({
      payload: {
        username
      },
      ...this
    });
    await context.loadPayload();
    return context;
  }
  /**
   * Get skin renders
   */


  getRenders(options) {
    return new _contexts.RendersContext({ ...this,
      ...options
    });
  }
  /**
   * Transform skin method
   */


  transformSkin({
    skin,
    transformation = "grayscale",
    model = _interfaces.Model.CLASSIC
  }) {
    return new _contexts.SkinContext({ ...this,
      payload: {
        hash: skin,
        model
      }
    }).transform(transformation);
  }
  /**
   * Get cape info by cape hash
   */


  getCapeInfo(hash) {
    return new _contexts.CapeContext({
      hash,
      ...this
    });
  }
  /**
   * Get player friends by nickname
   */


  async getFriends(username) {
    const endpoint = this.options.getEndpoint({
      domain: "api.ashcon.app"
    });
    const uuid = await (0, _utils.getUUID)(endpoint, username);
    return this.api.profile.friends({
      target: uuid
    });
  }
  /**
   * Get skins from a specific tab of the site
   */


  getSkins({
    tab,
    page = 1,
    section = "weekly"
  }) {
    return this.client.get(`/minecraft-skins/${tab}${tab === "trending" || tab === "tag" && section !== "weekly" ? `/${section}` : ""}`, {
      params: {
        page
      }
    }).then(({
      data
    }) => {
      const skins = this.parseSkins(data);

      if (!skins.length) {
        throw new _WrapperError.WrapperError("NO_USEFUL");
      }

      return skins;
    });
  }
  /**
   * Get information about skin
   */


  async getSkin(hash) {
    const skin = new _contexts.SkinContext({ ...this,
      payload: {
        hash
      }
    });
    await skin.loadPayload();
    return skin;
  }
  /**
   * Get servers list
   */


  getServers(page = 1) {
    return this.client.get(`/minecraft-servers/${page}`).then(({
      data
    }) => {
      var _servers$;

      const servers = this.parseServers(data);

      if (!((_servers$ = servers[0]) !== null && _servers$ !== void 0 && _servers$.ip)) {
        throw new _WrapperError.WrapperError("NO_USEFUL");
      }

      return servers;
    });
  }
  /**
   * Get server by ip
   */


  async getServer(ip) {
    const server = new _contexts.ServerContext({ ...this,
      payload: {
        ip
      }
    });
    await server.loadPayload();
    return server;
  }
  /**
   * Get server likes by ip
   */


  getServerLikes(ip) {
    return this.api.server.likes({
      target: ip
    });
  }
  /**
   * Check server like value by username
   */


  checkServerLike({
    ip,
    username
  }) {
    return new _contexts.ServerContext({ ...this,
      payload: {
        ip
      }
    }).checkLike(username);
  }
  /**
   * Search
   */


  search(query) {
    return this.client.get("/search", {
      params: {
        q: query
      }
    }).then(({
      request,
      headers,
      data
    }) => {
      var _request$res;

      const finalUrl = headers["x-final-url"] || (request === null || request === void 0 ? void 0 : (_request$res = request.res) === null || _request$res === void 0 ? void 0 : _request$res.responseUrl) || request.responseURL;

      if (finalUrl.match(_utils.profileRegExp)) {
        return new _contexts.PlayerContext({ ...this,
          data
        });
      }

      if (finalUrl.match(_utils.serverRegExp)) {
        return new _contexts.ServerContext({ ...this,
          data,
          extended: true
        });
      }

      return new _contexts.SearchContext({ ...this,
        data,
        payload: {
          query
        }
      });
    });
  }
  /**
   * @hidden
   */


  addInterceptors() {
    const {
      proxy,
      cloudProxy
    } = this.options;

    if (proxy && cloudProxy) {
      this.client.interceptors.request.use(config => {
        const {
          url,
          baseURL,
          method,
          data,
          params
        } = config;
        delete config.baseURL;
        delete config.params;
        const searchParams = params ? new URLSearchParams(Object.entries(params)).toString() : null;
        config.url = proxy;
        config.method = "post";
        config.data = { ...cloudProxy,
          url: `${baseURL}${url}${searchParams ? `?${searchParams}` : ""}`,
          cmd: `request.${method}`,
          headers: config.headers,
          postData: method === "post" ? data : undefined
        };
        return config;
      });
    }

    this.client.interceptors.response.use(response => {
      if (proxy && cloudProxy) {
        const {
          data: {
            solution: {
              url,
              response: htmlPage
            }
          }
        } = response;
        response.request.res.responseUrl = url;
        response.data = htmlPage;
      }

      return response;
    }, error => {
      var _error$response;

      switch (error === null || error === void 0 ? void 0 : (_error$response = error.response) === null || _error$response === void 0 ? void 0 : _error$response.status) {
        case 403:
          throw new _WrapperError.WrapperError("CLOUDFLARE");

        case 404:
          throw new _WrapperError.WrapperError("NOT_FOUND");

        default:
          throw error;
      }
    });
  }

}

exports.NameMC = NameMC;