"use strict";

exports.PlayerContext = void 0;

var cheerio = require("cheerio");

var _WrapperError = require("../WrapperError");

var _ = require("./");

var _utils = require("../utils");

var _interfaces = require("../interfaces");

const {
  FOLLOWING,
  FOLLOWERS
} = _interfaces.FollowersSection;

class PlayerContext extends _.Context {
  /**
   * @hidden
   */
  constructor({
    data,
    isSearch,
    ...options
  }) {
    super(options);
    /**
     * Profile ID
     */

    this.id = 0;
    /**
     * Player uuid
     */

    this.uuid = "";
    /**
     * Player username
     */

    this.username = "";
    /**
     * Profile short url
     */

    this.url = "";
    /**
     * Profile views
     */

    this.views = 0;
    /**
     * Nickname history
     */

    this.names = [];
    /**
     * Skin history
     */

    this.skins = [];
    /**
     * Player capes
     */

    this.capes = [];
    /**
     * Player friends
     */

    this.friends = [];
    /**
     * Player favorite servers
     */

    this.servers = [];
    /**
     * Player followers
     */

    this.followers = [];
    /**
     * Player following
     */

    this.following = [];
    /**
     * Following date, available when receiving player followers/following
     */

    this.followingDate = 0;
    /**
     * Badlion Client statistics
     *
     * @see {@link https://www.badlion.net/forum/thread/309559 | Announce from Badlion Client}
     */

    this.badlion = null;
    /**
     * Payload loaded
     * @hidden
     */

    this.extended = false;
    this.setupPayload();

    if (!data) {
      return;
    }

    const $ = cheerio.load(data);
    this.skins = $(`canvas.skin-2d${!isSearch ? ".skin-button" : ""}`) // @ts-ignore
    .map((index, {
      attribs: {
        "data-skin-hash": hash,
        "data-model": model = _interfaces.Model.UNKNOWN,
        title
      }
    }) => new _.SkinContext({ ...this,
      payload: {
        hash,
        model,
        createdAt: !isSearch ? (0, _utils.convertDate)(title) : 0
      }
    })).get();

    if (!isSearch) {
      this.capes = $("canvas.cape-2d").map((index, {
        attribs: {
          "data-cape-hash": hash
        }
      }) => new _.CapeContext({ ...this,
        hash
      })).get();
      this.servers = $("a > img.server-icon").map((index, element) => {
        var _element$parent, _element$parent$attri, _element$next;

        // @ts-ignore
        const ip = (_element$parent = element.parent) === null || _element$parent === void 0 ? void 0 : (_element$parent$attri = _element$parent.attribs) === null || _element$parent$attri === void 0 ? void 0 : _element$parent$attri.href.replace(_utils.serverRegExp, "$1"); // @ts-ignore

        const title = ((_element$next = element.next) === null || _element$next === void 0 ? void 0 : _element$next.data) || "";
        const {
          attribs: {
            src: icon
          }
        } = element;
        return new _.ServerContext({ ...this,
          extended: false,
          payload: {
            ip,
            title,
            icon
          }
        });
      }).get();
      const rawBadlionStatistic = $("div.card.badlion > div.card-body > div.row.no-gutters").get();

      if (rawBadlionStatistic.length === 4) {
        rawBadlionStatistic.splice(1, 0, {
          current: 0,
          max: 0
        });
      }

      const badlion = rawBadlionStatistic.map((element, index) => {
        const $ = cheerio.load(element);

        switch (index) {
          case 0:
            {
              return (0, _utils.parseDuration)($("div.col-12").text());
            }

          case 1:
            {
              if (element !== null && element !== void 0 && element.current) {
                return element;
              }

              const [current, max] = $("div.col-12").text().match(/([\d]+)/g) || [0, 0];
              return {
                current: Number(current),
                max: Number(max)
              };
            }

          case 3:
            {
              var _time$attribs;

              const time = $("div.col-12 > time").get(0);
              const lastOnline = (time === null || time === void 0 ? void 0 : (_time$attribs = time.attribs) === null || _time$attribs === void 0 ? void 0 : _time$attribs.datetime) || null;
              return lastOnline ? new Date(lastOnline).getTime() : null;
            }

          case 2:
          case 4:
            {
              return $("div.col-12").text();
            }
        }
      });
      const [baseInfoRaw, usernameHistoryRaw] = $("div.card.mb-3 > div.card-body").map((index, element) => {
        const $ = cheerio.load(element);
        const body = $("div.card-body");

        switch (index) {
          case 0:
            return body.children("div.row.no-gutters");

          case 1:
            return body.find("tr:not([class])").map((index, element) => {
              element.children.push(element.next);
              return element;
            });
        }
      }).get();
      const baseInfo = baseInfoRaw.map((index, element) => {
        const $ = cheerio.load(element);

        switch (index) {
          case 0:
          case 1:
            return $("div.col-12 > samp").text();

          case 2:
            {
              const link = $("div.col-12 > a").text();
              return `https://${link}`;
            }

          case 3:
            {
              const views = parseInt($("div.col-auto").text());
              return Number(views);
            }
        }
      }).get();
      const usernameHistory = this.parseUsernameHistory(usernameHistoryRaw);
      const [uuid,, url, views] = baseInfo;
      this.id = Number(url.split(".").pop());
      this.uuid = uuid;
      this.url = url;
      this.views = views;
      this.names = usernameHistory;

      if (badlion.length) {
        const [play_time, login_streak, last_server, last_online, version] = badlion;
        this.badlion = {
          play_time,
          login_streak,
          last_server,
          last_online,
          version
        };
      }
    } else {
      const [,, id] = $("div.card-header a").get(0).attribs.href.match(_utils.profileRegExp);
      this.id = Number(id);
      this.names = this.parseUsernameHistory($("tr"));
    }

    this.username = this.names[this.names.length - 1].username;
  }
  /**
   * Check payload loaded
   */


  get isExtended() {
    return this.extended;
  }
  /**
   * Does the player use Badlion Client
   */


  get usesBadlion() {
    return Boolean(this.badlion);
  }
  /**
   * Get clean UUID
   */


  get cleanUUID() {
    return this.uuid.replace(/-/g, "");
  }
  /**
   * Load all profile information
   */


  async loadPayload() {
    if (this.isExtended) {
      return;
    }

    if (!(this.username && this.uuid && this.url && this.id)) {
      const username = this.uuid || this.username;

      if (!username.match(_utils.nameRegExp)) {
        throw new _WrapperError.WrapperError("INVALID_NICKNAME", username);
      }

      await this.client.get(`/profile/${username}`).then(({
        request,
        headers,
        data
      }) => {
        var _request$res;

        if ((headers["x-final-url"] || (request === null || request === void 0 ? void 0 : (_request$res = request.res) === null || _request$res === void 0 ? void 0 : _request$res.responseUrl) || request.responseURL).match(_utils.profileRegExp)) {
          this.payload = new PlayerContext({
            data,
            ...this
          }).toJSON();
          return this.setupPayload();
        }

        throw new _WrapperError.WrapperError("NOT_FOUND", username);
      });
    }

    this.payload = await this.api.profile.friends({
      target: this.uuid
    }).then(friends => ({
      friends
    }));
    this.setupPayload();
    this.extended = true;
  }
  /**
   * Get player followers
   */


  getFollowers(options) {
    return this.loadFollowers({ ...options,
      section: FOLLOWERS
    });
  }
  /**
   * Get player following
   */


  getFollowing(options) {
    return this.loadFollowers({ ...options,
      section: FOLLOWING
    });
  }
  /**
   * @hidden
   */


  async loadFollowers({
    section,
    page = 1,
    sort = {}
  }) {
    if (!this.extended) {
      await this.loadPayload();
    }

    const filter = Object.entries(sort).map(filter => filter.join(":")).join(",");
    const loadedFollowers = await this.client.get(`/profile/${this.uuid}/${section}`, {
      params: {
        page,
        sort: filter
      }
    }).then(({
      data
    }) => this.parseFollowers(data));
    const followers = loadedFollowers.reduce((acc, follower) => {
      if (!acc.filter(({
        username
      }) => username === follower.username).length) {
        acc.push(follower);
      }

      return acc;
    }, this[section]);
    this.payload = {
      [section]: followers
    };
    this.setupPayload();
    return loadedFollowers;
  }
  /**
   * @hidden
   */


  parseFollowers(data) {
    const $ = cheerio.load(data);
    const uuid = $("tbody > input[name='profile']").map((index, {
      attribs: {
        value: uuid
      }
    }) => uuid).get();
    const names = $("tbody > tr").map((index, element) => {
      const $ = cheerio.load(element.children);
      const children = $("td");
      const name = cheerio.load(children.get(2));
      const username = name.text(); // @ts-ignore

      const {
        attribs: {
          datetime
        }
      } = children.get(3).firstChild;
      return {
        username,
        date: new Date(datetime).getTime()
      };
    }).get();
    return names.map(({
      username,
      date: followingDate
    }, index) => new PlayerContext({ ...this,
      payload: {
        username,
        followingDate,
        uuid: uuid[index]
      }
    }));
  }
  /**
   * @hidden
   */


  parseUsernameHistory(element) {
    return element.map((index, element) => {
      const $ = cheerio.load(element);
      const name = $("a").get(0);
      const time = $("time").get(0);

      if (name) {
        var _time$attribs2;

        // @ts-ignore
        const {
          children: [{
            data: username
          }]
        } = name;
        const changed_at = (time === null || time === void 0 ? void 0 : (_time$attribs2 = time.attribs) === null || _time$attribs2 === void 0 ? void 0 : _time$attribs2.datetime) || null;
        return {
          username,
          changed_at,
          timestamp: changed_at ? new Date(changed_at).getTime() : null
        };
      }
    }).get().reverse();
  }
  /**
   * @hidden
   */


  [_utils.kSerializeData]() {
    return (0, _utils.pickProperties)(this, ["id", "uuid", "username", "url", "views", "names", "skins", "capes", "friends", "followers", "following", "followingDate", "badlion", "servers"]);
  }

}

exports.PlayerContext = PlayerContext;