"use strict";

exports.SearchContext = void 0;

var cheerio = require("cheerio");

var _context = require("./context");

var _skin = require("./skin");

var _player = require("./player");

var _server = require("./server");

var _utils = require("../utils");

var _interfaces = require("../interfaces");

class SearchContext extends _context.Context {
  /**
   * @hidden
   */
  constructor({
    data,
    ...options
  }) {
    super(options);
    /**
     * Search Query
     */

    this.query = "";
    /**
     * Query name info
     */

    this.name = {
      /**
       * Name status
       */
      status: _interfaces.NameStatus.INVALID,

      /**
       * Name availability timestamp
       */
      availabilityTime: null,

      /**
       * Name availability time
       */
      availabilityAt: null,

      /**
       * Name searches
       */
      searches: 0
    };
    /**
     * Query players
     */

    this.players = [];
    /**
     * Query servers
     */

    this.servers = [];
    /**
     * Query skin tag
     */

    this.skin = null;
    this.setupPayload();

    if (!data) {
      return;
    }

    const $ = cheerio.load(data);
    const [skinTagCol, mainCol] = $("main > div > div.col-12").map((index, element) => cheerio.load(element)).get();
    const skin = skinTagCol("div.card-body").get(0);

    if (skin) {
      this.skin = new _skin.SkinContext({ ...this,
        data: skin
      });
    }

    mainCol("div.card.mb-3, div:not(.ad-container).mb-3").get().forEach(element => {
      const $ = cheerio.load(element);
      const statusBar = $("div#status-bar");

      if (statusBar.length) {
        this.name.status = statusBar.hasClass("bg-success") ? _interfaces.NameStatus.AVAILABLE : statusBar.hasClass("bg-info") ? _interfaces.NameStatus.AVAILABLE_LATER : statusBar.hasClass("bg-warning") ? _interfaces.NameStatus.UNAVAILABLE : _interfaces.NameStatus.INVALID;
        const availabilityTime = statusBar.find("time");

        if (availabilityTime.length) {
          this.name.availabilityAt = availabilityTime.attr().datetime;
          this.name.availabilityTime = new Date(this.name.availabilityAt).getTime();
        }

        this.name.searches = parseInt(statusBar.find("div.tabular").text());
        return;
      }

      const servers = $("div.table-responsive tr");

      if (servers.length) {
        this.servers.push(...servers.map((index, element) => new _server.ServerContext({ ...this,
          data: element,
          isSearch: true
        })).get());
        return;
      }

      this.players.push(new _player.PlayerContext({ ...this,
        data: $("div.card").get(0),
        isSearch: true
      }));
    });
  }
  /**
   * Check is query name is available
   */


  get isAvailable() {
    return this.name.status === _interfaces.NameStatus.AVAILABLE;
  }
  /**
   * Check is query name is available available later
   */


  get isAvailableLater() {
    return this.name.status === _interfaces.NameStatus.AVAILABLE_LATER;
  }
  /**
   * Check is query name is unavailable
   */


  get isUnavailable() {
    return this.name.status === _interfaces.NameStatus.UNAVAILABLE;
  }
  /**
   * Check is query name is invalid
   */


  get isInvalid() {
    return this.name.status === _interfaces.NameStatus.INVALID;
  }
  /**
   * Check query has players
   */


  get hasPlayers() {
    return Boolean(this.players.length);
  }
  /**
   * Check query has servers
   */


  get hasServers() {
    return Boolean(this.servers.length);
  }
  /**
   * Check query has skin tag name
   */


  get hasSkinTag() {
    return Boolean(this.skin);
  }
  /**
   * @hidden
   */


  [_utils.kSerializeData]() {
    return (0, _utils.pickProperties)(this, ["query", "name", "players", "servers", "skin"]);
  }

}

exports.SearchContext = SearchContext;