"use strict";

exports.SkinContext = void 0;

var cheerio = require("cheerio");

var _context = require("./context");

var _renders = require("./renders");

var _WrapperError = require("../WrapperError");

var _utils = require("../utils");

var _interfaces = require("../interfaces");

class SkinContext extends _context.Context {
  /**
   * @hidden
   */
  constructor({
    data,
    type,
    ...options
  }) {
    super(options);
    /**
     * Skin hash
     */

    this.hash = "";
    /**
     * Skin name
     */

    this.name = "";
    /**
     * Skin model
     */

    this.model = _interfaces.Model.UNKNOWN;
    /**
     * Skin tags
     */

    this.tags = [];
    /**
     * Skin rating
     */

    this.rating = 0;
    /**
     * Skin creation timestamp
     */

    this.createdAt = 0;
    /**
     * Skin transformation type
     */

    this.transformation = null;
    /**
     * Payload loaded
     * @hidden
     */

    this.extended = false;
    this.setupPayload();

    if (!data) {
      return;
    }

    switch (type) {
      case "extended":
        {
          this.extended = true;
          const $ = cheerio.load(data);
          const {
            attribs: {
              href
            }
          } = $("#render-button.btn").get(0);
          const isValidSkin = this.checkSkinLink(href);

          if (isValidSkin) {
            const {
              hash,
              model
            } = isValidSkin;
            this.hash = hash;
            this.model = model;
            this.rating = this.parseSkinRating($);
            this.createdAt = this.parseSkinTime($);
            this.tags = $("div.card-body.text-center.py-1 > a.badge.badge-pill").map((index, element) => {
              // @ts-ignore
              const {
                children: [{
                  data: tag
                }]
              } = element;
              return tag;
            }).get();
          }

          break;
        }

      default:
        {
          var _data$parent, _data$parent2, _cardHeader$children$;

          // @ts-ignore
          const cardLinkHash = data === null || data === void 0 ? void 0 : (_data$parent = data.parent) === null || _data$parent === void 0 ? void 0 : _data$parent.attribs.href.replace(_utils.skinRegExp, "$1");
          const cardHeader = data === null || data === void 0 ? void 0 : (_data$parent2 = data.parent) === null || _data$parent2 === void 0 ? void 0 : _data$parent2.children // @ts-ignore
          .filter(({
            name,
            attribs: {
              class: className = ""
            } = {}
          }) => name === "div" && className.includes("card-header"))[0];
          const cardName = cardHeader ? // @ts-ignore
          (_cardHeader$children$ = cardHeader.children[0]) === null || _cardHeader$children$ === void 0 ? void 0 : _cardHeader$children$.data : "";
          const $ = cheerio.load(data);
          const [{
            hash,
            model
          }] = $("div > img.drop-shadow").map((index, {
            attribs: {
              "data-src": src
            }
          }) => {
            const isValidSkin = this.checkSkinLink(src);
            return {
              model: _interfaces.Model.UNKNOWN,
              hash: cardLinkHash,
              ...isValidSkin
            };
          }).get();
          this.model = model;
          this.hash = hash;
          this.name = cardName;
          this.rating = this.parseSkinRating($);
        }
    }
  }
  /**
   * Get skin url
   */


  get url() {
    return `${this.options.getEndpoint()}/texture/${this.hash}.png`;
  }
  /**
   * Get skin renders
   */


  get renders() {
    const {
      options,
      client,
      api
    } = this;
    return new _renders.RendersContext({
      skin: this.hash,
      model: this.model,
      options,
      client,
      api
    });
  }
  /**
   * Check is slim skin model
   */


  get isSlim() {
    return this.model === _interfaces.Model.SLIM;
  }
  /**
   * Check payload loaded
   */


  get isExtended() {
    return this.extended;
  }
  /**
   * Check is transformed
   */


  get isTransformed() {
    return Boolean(this.transformation);
  }
  /**
   * Transform skin
   *
   * @see {@link https://namemc.com/skin/ee40191789e621d3 | Check "Tools" card}
   */


  transform(transformation) {
    return this.client.post("/transform-skin", `skin=${this.hash}&transformation=${transformation}`, {
      headers: {
        "Content-Type": "application/x-www-form-urlencoded",
        origin: "https://ru.namemc.com"
      }
    }).then(({
      request
    }) => {
      var _request$res;

      const [, hash] = ((request === null || request === void 0 ? void 0 : (_request$res = request.res) === null || _request$res === void 0 ? void 0 : _request$res.responseUrl) || request.responseURL).match(_utils.skinRegExp);

      if (!hash) {
        throw new _WrapperError.WrapperError("NO_USEFUL");
      }

      this.payload = {
        hash,
        transformation
      };
      this.setupPayload();
      return this;
    }).catch(error => {
      var _error$response;

      throw (error === null || error === void 0 ? void 0 : (_error$response = error.response) === null || _error$response === void 0 ? void 0 : _error$response.status) === 404 ? new _WrapperError.WrapperError("NOT_FOUND", this.hash) : error;
    });
  }
  /**
   * Load all skin information
   */


  async loadPayload() {
    if (this.isExtended) {
      return;
    }

    this.payload = await this.client.get(`/skin/${this.hash}`).then(({
      data
    }) => {
      const skin = new SkinContext({ ...this,
        data,
        type: "extended",
        payload: {
          name: this.name
        }
      });

      if (!skin.hash) {
        throw new _WrapperError.WrapperError("NO_USEFUL");
      }

      return skin;
    });
    this.setupPayload();
    this.extended = true;
  }
  /**
   * @hidden
   */


  checkSkinLink(link) {
    const skinRegExp = /skin=([^]+?)(?:&[^]+)?&model=([^]+?)&[^]+/;
    const idValidSkin = skinRegExp.exec(link);

    if (idValidSkin) {
      const [, hash, model] = idValidSkin;
      return {
        hash,
        model: model
      };
    }
  }
  /**
   * @hidden
   */


  parseSkinRating($) {
    var _$$get;

    const ratingElement = ((_$$get = $(".position-absolute.bottom-0.right-0.text-muted").get(0)) === null || _$$get === void 0 ? void 0 : _$$get.children) || null;
    const rating = ratingElement ? (ratingElement.length > 1 ? ratingElement[1] : ratingElement[0]). // @ts-ignore Invalid lib types
    data : "0";
    return Number(rating.replace(/[^\d]+([\d]+)/, "$1"));
  }
  /**
   * @hidden
   */


  parseSkinTime($) {
    const date = $(".position-absolute.bottom-0.left-0.text-muted.title-time").get(0).attribs.title;
    return new Date(date).getTime();
  }
  /**
   * @hidden
   */


  [_utils.kSerializeData]() {
    return (0, _utils.pickProperties)(this, ["hash", "name", "url", "model", "isSlim", "transformation", "isTransformed", "renders", "tags", "rating", "createdAt"]);
  }

}

exports.SkinContext = SkinContext;